LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;

PACKAGE datos IS
	CONSTANT width	:	INTEGER := 8;
	TYPE datos IS ARRAY(1 to width) OF std_logic;
END datos;

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE WORK.datos.ALL;

ENTITY m232c IS
	PORT ( 	
			p1 : IN std_logic;			-- Received Data desde el punto de vista del dispositivo
			p3 : OUT std_logic;			-- Transmited data, idem.
			envia : IN std_logic;		-- seal para activar el envio.
			rebut : OUT std_logic;		-- seal auxiliar que indica la recepcin de un byte.
			baud : IN std_logic;		-- clk de trabajo. En nuestros caso 9600 baudios
			dadain : IN datos;			-- buffer de entrada de datos
			dadaout : OUT datos;		-- buffer de salida. conectados externamente reenviaremos el byte que llegue.
			p2 : IN std_logic;			-- RTS. Request to send desde el punto de vista del emisor.
										-- para el dispositivo su CTS.
			p5 : OUT std_logic;			-- CTS. Clear to send desde elpunto de vista del emisor.
										-- para el dispositivo su RTS.
			paridad : IN std_logic;		-- '1' indica con paridad. '0' sin paridad
			oddeven : IN std_logic);	-- paridad par '0' o impar '1'
END m232c;

ARCHITECTURE arq_s232 OF m232c IS
	SIGNAL estat : INTEGER RANGE 0 to 3;	-- estados de recepcion
	SIGNAL estaten : INTEGER RANGE 0 to 6;	-- estados de envio
	SIGNAL pari,pari2 : std_logic;			-- pari es el calculo de la paridad al recibir, pari2 al enviar.
	SIGNAL i,i2 : INTEGER RANGE 1 TO 8;		-- i controla los bits recibidos. i2 los bits enviados.
	signal p5ctrl, p3ctrl : std_logic;		-- seales para controlar la salida o dejar la linea en alta impedancia.
	signal p5value, p3value, enviar : std_logic;	-- valores que se asignarn a las seales de salida cuando esten bajo control.

BEGIN
	PROCESS(envia)
	BEGIN
		IF estaten = 6 then 	-- el 6 es el ultimo estado del envio, para volver a recibir
			enviar <= '0';
		ELSIF rising_edge(envia) then	-- con la seal envia se activa el modo de envio.
			enviar <= '1';
		END IF;
	END PROCESS;
		
	PROCESS(baud)
	BEGIN
		IF rising_edge(baud) THEN
			if enviar = '0' then
				CASE estat IS
					WHEN 0 =>
						rebut <= '0';
						IF p2 = '0' THEN		-- cuando recibimos el RTS p2 = '0' ponemos el CTS tambien a '0'
							p5value <= '0';		-- recordar que el 0 representa activo.
							p5ctrl <= '0';
							IF p1 = '0' THEN 	-- esperamos el bit de start antes de pasar a recibir los datos.
								estat <= 1;		
								i <= 1;			
								pari <= '0';	-- inicializamos la paridad 
							END IF;
						ELSE
							p5value <= '1';		-- si se levanta el RTS levantamos el CTS
							p5ctrl <= '0';
						END IF;
					WHEN 1 =>
						pari <= pari XOR p1;
						IF i < 8 THEN			-- recepcion de los ocho bits a la vez que se calcula la paridad
							dadaout(i) <= p1;	
							i <= i + 1;
						ELSIF i = 8 THEN 
							i <= 1;
							IF paridad = '0' then 			-- si no hay paridad se salta al bit de stop.
								estat <= 2;					
							ELSIF paridad = '1' then		-- si hay paridad se estudia en el estado 3.
								estat <= 3; 
							ELSE estat <= 0;
							END IF;
						END IF;
					WHEN 2 =>
						IF p1 = '1' THEN rebut <= '1';		-- esperamos el bit de stop. la seal rebut se conecta
						END IF;								-- externamente con la seal envia para que proceda al envio.
						estat <= 0;							-- del mismo valor y se vuelve al estado inicial.
					WHEN 3 =>
						IF oddeven = '0' then 
							IF p1 /= pari THEN estat <= 0; 	-- si oddeven es '0', indica even, 
															-- tendrn que coincidir la recibida y la calculada
							ELSE estat <= 2;				-- para pasar a recibir el bit de stop del estado 2
							END IF;
						ELSIF oddeven = '1' then 			-- si oddeven es '1', indica evodden, 
															-- no tendrn que coincidir la recibida y la calculada
							IF p1 = pari THEN estat <= 0;	-- para pasar a recibir el bit de stop del estado 2
							ELSE estat <= 2;
							END IF;
						END IF;
				END CASE;
				end if;
				
				IF enviar = '1' then
					case estaten is
						when 0 =>
							p5value <= '0';				-- bajamos el CTS, para nosotros el RTS.
							p5ctrl <= '0';
							estaten <= 1;
						when 1 =>
							IF p2 = '0' THEN
							pari2 <= '0';	
							p3value <= '0';				-- Se envia el bit start.
							p3ctrl <= '0';
							estaten <= 2;
							i2 <= 1;
							END IF;
						when 2 =>
							pari2 <= pari2 XOR p1;		-- se calcula la paridad.
							p3value <= dadain(i2);		-- se envian los datos.
							p3ctrl <= '0';
							i2 <= i2 + 1;
							IF i2 = 8 then
								IF paridad = '0' then	-- tras el 8 bit se va al estado 3 o al 5.
									estaten <= 3;		-- el 3 envia el bit de stop, cuando no hay paridad.
								ELSIF paridad = '1' then	-- el estado 5 envia la paridad.
									estaten <= 5;
								ELSE estaten <= 4;		-- error, no se da.
								END IF;
							END IF;
						when 3 =>
							p3value <= '1';				-- se envia el bit stop.
							p3ctrl <= '0';
							estaten <= 4;
						when 4 =>
							estaten <= 6;
						when 5 =>
							IF oddeven = '0' then 		-- si la paridad es par se envia el bit tal cual,
								p3value <= pari2;
							ELSIF oddeven = '1' then 	-- si es impar se invierte.
								p3value <= pari2 XOR '1';
							END IF;
							p3ctrl <= '0';
							estaten <= 3;
						when 6 =>
							null;
						end case;
				else
					estaten<=0;
				END IF;
				
		END IF;
	END PROCESS;
	
	process
	begin
		if p5ctrl='1' then				-- cuando pXctrl es uno se deja la linea en alta impedancia
			p5 <= 'Z';					-- en caso contrario se coloca el valor de PXvalue.
		else
			p5 <= p5value;				
		end if;
		if p3ctrl='1' then
			p3 <= 'Z';
		else
			p3 <= p3value;
		end if;
	end process;
END arq_s232;

