LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;
USE WORK.datos.ALL;

ENTITY CtrlUnit45 IS
	PORT ( 	
			vtrim : IN std_logic;		-- seal que proviene del chip indicando que el lado USB est alimentado.
			clk : IN std_logic;			-- seal de clock a 12 Mhz a partir del clock interno de la placa 

			length : IN integer RANGE 0 TO 127;				-- longitud del paquete a enviar en la fase de datos.	
			
			-- Durante la recepcin de un paquete de datos, en la fase de setup de una transmisin de control,
			-- se descompone este en sus partes.
			
			bmRequestType : IN std_logic_vector(0 to 7);	-- Tipo de peticin. primer byte del paquete de datos
			bRequest : IN std_logic_vector(0 to 7);			-- Peticin en particular. segundo byte del paquete.
			wValue : IN std_logic_vector(0 to 15);			-- 3er y 4o bytes. Varia en funcion de la peticin.
															-- alternayivamente se usa para guardar la direccion 
															-- destino de un paquete de testigo.
			wIndex : IN std_logic_vector(0 to 15);			-- 5o y 6o bytes. tb varia segn la peticin wIndex y wValue
															-- normalmente son parametros de indice o offset.
			wLength : IN std_logic_vector(0 to 15);			-- longitud de los datos que se transferirn en la segunda 
															-- fase de la transmisin de control, la seal length es su 
															-- valor numerico.
			reset : IN std_logic;							-- seal de reset recibido.
			pide : IN std_logic_vector(0 to 7);				-- PID del packete recibido. El PID indica su tipo.
			pr : IN std_logic;								-- seal de paquete recibido.
			crc : IN std_logic;								-- indica si la comprobacin del crc ha sido exitosa valor 1 
															-- o no 0. 
															-- (seal HSD2_CRC16 en el diagrama de estados de la especificacin).
			rsac : OUT std_logic;							-- seal de activacion del buffer para el envio de datos.
			pids : OUT std_logic_vector(0 to 7);			-- PID a enviar.
			long_dades : OUT INTEGER RANGE 0 TO 255;		-- longitud de los datos a enviar.
			buffer_salida : OUT std_logic_vector(0 to 255);	-- Buffer de datos a enviar.
			led_1 : OUT std_logic;							-- led_1 de la placa.
			led_2 : OUT std_logic;							-- led_2 de la placa.
			led_3 : OUT std_logic;							-- led_3 de la placa.
			led_4 : OUT std_logic							-- led_4 de la placa.			
		);
	
	-- Todos los PIDs posibles. los 4 primeros bits son las 16 posibles combinaciones, los 4 segundos son 
	-- los primeros pero invertidos y sirven para la comprobacion de errores. (se duplican los datos).

	CONSTANT token_out	:	std_logic_vector(0 to 7) := ('1','0','0','0','0','1','1','1');
	CONSTANT token_in	:	std_logic_vector(0 to 7) := ('1','0','0','1','0','1','1','0');	
	CONSTANT sof		:	std_logic_vector(0 to 7) := ('1','0','1','0','0','1','0','1');
	CONSTANT setup		:	std_logic_vector(0 to 7) := ('1','0','1','1','0','1','0','0');
	CONSTANT data0		:	std_logic_vector(0 to 7) := ('1','1','0','0','0','0','1','1');
	CONSTANT data1		:	std_logic_vector(0 to 7) := ('1','1','0','1','0','0','1','0');
	CONSTANT data2		:	std_logic_vector(0 to 7) := ('1','1','1','0','0','0','0','1');
	CONSTANT mdata		:	std_logic_vector(0 to 7) := ('1','1','1','1','0','0','0','0');
	CONSTANT ackk		:	std_logic_vector(0 to 7) := ('0','1','0','0','1','0','1','1');
	CONSTANT nack		:	std_logic_vector(0 to 7) := ('0','1','0','1','1','0','1','0');
	CONSTANT stall		:	std_logic_vector(0 to 7) := ('0','1','1','1','1','0','0','0');
	CONSTANT nyet		:	std_logic_vector(0 to 7) := ('0','1','1','0','1','0','0','1');
	CONSTANT err		:	std_logic_vector(0 to 7) := ('0','0','1','1','1','1','0','0');
	CONSTANT split		:	std_logic_vector(0 to 7) := ('0','0','0','1','1','1','1','0');
	CONSTANT ping		:	std_logic_vector(0 to 7) := ('0','0','1','0','1','1','0','1');
	CONSTANT reserved	:	std_logic_vector(0 to 7) := ('0','0','0','0','1','1','1','1');		
	
	-- direccin por defecto o de la DEFAULT_PIPE. 

	CONSTANT DEFAULT_DIR		: 	std_logic_vector(0 to 10) := ('0','0','0','0','0','0','0','0','0','0','0');

	-- Peticiones Standard. brequest posibles.
	
	CONSTANT GET_STATUS 		: 	std_logic_vector(0 to 7) := ('0','0','0','0','0','0','0','0');		-- Standard Request Code Value 0
	CONSTANT CLEAR_FEATURE		: 	std_logic_vector(0 to 7) := ('1','0','0','0','0','0','0','0');		-- Standard Request Code Value 1
																										-- El 2 reservado para uso futuro
	CONSTANT SET_FEATURE		: 	std_logic_vector(0 to 7) := ('1','1','0','0','0','0','0','0');		-- Standard Request Code Value 3
																										-- El 4 reservado para uso futuro
	CONSTANT SET_ADDRESS 		: 	std_logic_vector(0 to 7) := ('1','0','1','0','0','0','0','0');		-- Standard Request Code Value 5
	CONSTANT GET_DESCRIPTOR		: 	std_logic_vector(0 to 7) := ('0','1','1','0','0','0','0','0');		-- Standard Request Code Value 6
	CONSTANT SET_DESCRIPTOR 	: 	std_logic_vector(0 to 7) := ('1','1','1','0','0','0','0','0');		-- Standard Request Code Value 7
	CONSTANT GET_CONFIGURATION 	: 	std_logic_vector(0 to 7) := ('0','0','0','1','0','0','0','0');		-- Standard Request Code Value 8
	CONSTANT SET_CONFIGURATION 	: 	std_logic_vector(0 to 7) := ('1','0','0','1','0','0','0','0');		-- Standard Request Code Value 9
	CONSTANT GET_INTERFACE 		: 	std_logic_vector(0 to 7) := ('0','1','0','1','0','0','0','0');		-- Standard Request Code Value 10
	CONSTANT SET_INTERFACE 		: 	std_logic_vector(0 to 7) := ('1','1','0','1','0','0','0','0');		-- Standard Request Code Value 11
	CONSTANT SYNCH_FRAME	 	: 	std_logic_vector(0 to 7) := ('0','0','1','1','0','0','0','0');		-- Standard Request Code Value 12	

	-- Tipos de peticin Standard. bmRequestType posibles.
	
	CONSTANT SETZERO	:	std_logic_vector(0 to 7) := ('0','0','0','0','0','0','0','0');	
	CONSTANT SETUNO 	:	std_logic_vector(0 to 7) := ('1','0','0','0','0','0','0','0');	
	CONSTANT SETDOS 	:	std_logic_vector(0 to 7) := ('0','1','0','0','0','0','0','0');	
	CONSTANT GETZERO 	:	std_logic_vector(0 to 7) := ('0','0','0','0','0','0','0','1');	
	CONSTANT GETUNO 	:	std_logic_vector(0 to 7) := ('1','0','0','0','0','0','0','1');	
	CONSTANT GETDOS 	:	std_logic_vector(0 to 7) := ('0','1','0','0','0','0','0','1');	

	-- Valores que se usan para rellenar los diferentes descriptores.

	CONSTANT INT00IN	:	std_logic_vector(0 to 7) := ('0','0','0','0','0','0','0','0');	
	CONSTANT INT01IN	:	std_logic_vector(0 to 7) := ('1','0','0','0','0','0','0','0');	
	CONSTANT INT02IN	:	std_logic_vector(0 to 7) := ('0','1','0','0','0','0','0','0');		
	CONSTANT INT03IN	:	std_logic_vector(0 to 7) := ('1','1','0','0','0','0','0','0');		
	CONSTANT INT04IN  	: 	std_logic_vector(0 to 7) := ('0','0','1','0','0','0','0','0');		
	CONSTANT INT05IN	:	std_logic_vector(0 to 7) := ('1','0','1','0','0','0','0','0');		
	CONSTANT INT07IN	:	std_logic_vector(0 to 7) := ('1','1','1','0','0','0','0','0');		
	CONSTANT INT09IN	:	std_logic_vector(0 to 7) := ('1','0','0','1','0','0','0','0');		
	CONSTANT INT10IN	:	std_logic_vector(0 to 7) := ('0','0','0','0','1','0','0','0');	
	CONSTANT INT32IN	:	std_logic_vector(0 to 7) := ('0','0','0','0','0','1','0','0');
	CONSTANT INT18IN	:	std_logic_vector(0 to 7) := ('0','1','0','0','1','0','0','0');	
	CONSTANT INT40IN	:  std_logic_vector(0 to 7) := ('0','0','0','0','0','0','1','0');	

	CONSTANT ZERO15		:	std_logic_vector(0 to 15) := ('0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0');		
	CONSTANT UNO15		:	std_logic_vector(0 to 15) := ('0','0','0','0','0','0','0','0','1','0','0','0','0','0','0','0');		
	CONSTANT DOS15		:	std_logic_vector(0 to 15) := ('0','0','0','0','0','0','0','0','0','1','0','0','0','0','0','0');		
	CONSTANT TRES15			:	std_logic_vector(0 to 15) := ('0','0','0','0','0','0','0','0','1','1','0','0','0','0','0','0');		
	CONSTANT CUATRO15		:	std_logic_vector(0 to 15) := ('0','0','0','0','0','0','0','0','0','0','1','0','0','0','0','0');		
	CONSTANT CINCO15		:	std_logic_vector(0 to 15) := ('0','0','0','0','0','0','0','0','1','0','1','0','0','0','0','0');		
	CONSTANT SEIS15			:	std_logic_vector(0 to 15) := ('0','0','0','0','0','0','0','0','0','1','1','0','0','0','0','0');		
	CONSTANT SIETE15		:	std_logic_vector(0 to 15) := ('0','0','0','0','0','0','0','0','1','1','1','0','0','0','0','0');						
	CONSTANT OCHO15			:	std_logic_vector(0 to 15) := ('0','0','0','0','0','0','0','0','0','0','0','1','0','0','0','0');		

	-- Valores para el bEndpointAddress de los descriptores de Endpoint.
	
	CONSTANT EP1	:	std_logic_vector(0 to 7) := ('1','0','0','0','0','0','0','1');	
	CONSTANT EP2	:	std_logic_vector(0 to 7) := ('0','1','0','0','0','0','0','0');	
	
	-- Estados generales en los que se pueden encontrar el dispositivo.
	-- El estado suspendido se considera en la capa fsica.
	
	TYPE dev_states IS (attached,powered,default,address,configured);	
	
	-- Tipos de Endpoint. segn el tipo de transferencia
	
	TYPE endpoints IS (ep_control, ep_bulk, ep_interrupt);				
	
	-- Estados posibles durante una transmisin de control
	
	TYPE makina_estats IS (packet_ready, dev_wait_Odata, dev_resp);
	
	-- Fases o etapas de una transmisin de control
	
	TYPE fases_ctrl is (setup_ctrl,datain_ctrl,dataout_ctrl,status_ctrl);
	
END CtrlUnit45;

ARCHITECTURE arq_CtrlUnit45 OF CtrlUnit45 IS
	
	SIGNAL estat_act : dev_states;					-- variable con el estado general en el que se encuentra el dispositivo
	SIGNAL token_pid : std_logic_vector(0 to 7);	-- PID del paquete anterior al actual, sirve para controlar la fase dentro 
													-- de la transaccin.
	SIGNAL ep_type : endpoints;						-- tipo de endpoint actual. De momento siempre ep_control.
	SIGNAL space_avail : std_logic;					-- Indica si el dispositivo esta preparado para recibir mas datos. de momento no se usa.
	SIGNAL toggle : std_logic;						-- Indica el tipo de paquete de dtos que toca. Se alternan Data0 y Data1.
	SIGNAL ep_trouble : std_logic;					-- Indica si ha habido un problema en la transmisin. Se devuelve el paquete de STALL.
	SIGNAL data_avail : std_logic;					-- Indica si los datos estan preparados para su envio.
	SIGNAL timeout : std_logic;						-- Controla el tiempo por si se tarda demasiado en recibir un paquete. No se usa.
	
	SIGNAL dentro : std_logic; 						-- se pone a uno para entrar en la segunda maquina de estados que se efectua 
													-- en un flanco de subida. Sirve para inicializar variables.
	
	SIGNAL direccion_act : std_logic_vector(0 to 10);	-- Direccin actual del dispositivo.
	SIGNAL dir_valida : std_logic;						-- Indica desde la mquina de la transaccin cuando se ha realizado con exito un 
														-- direccionamiento. La primera mquina pondr el dispositivo en estado addressed
	SIGNAL dir_no_valida : std_logic;					-- Indica cuando se ha reseteado un dispositivo por una transacion. Lo mismo 
														-- que el reset pero a travs de un paquete al dispositivo ya direccionado.
	SIGNAL configurado : std_logic;						-- Indica a la primera mquina que el dispositivo a sido configurado.
	SIGNAL no_configurado : std_logic;					-- Indica a la primera mquina que el dispositivo a sido desconfigurado.
	SIGNAL mak_est 	: makina_estats;					-- Estado del dispositivo en una etapa de la transaccin. En funcin de que est 
														-- esperando en ese momento.
	SIGNAL primer : std_logic;							-- Para controlar un caso especial al inicio en el que mientras se envia un paquete 
														-- se recibe un reset.
	SIGNAL fasesc : fases_ctrl;							-- fase del dispositivo dentro de una transaccin de control.
	SIGNAL anterior : std_logic_vector(0 to 7);			-- El direccionamiento no se efectua hasta el final de la etapa de estatus aunque se
														-- conozca de antemano.
	SIGNAL lengthw : integer RANGE 0 TO 127;			-- variable auxiliar a la longitud de salida de datos para poder asignarla en la 
														-- siguiente etapa.
	SIGNAL configuracion : std_logic_vector(0 to 7);	-- configuracin actual seleccionada dentro de las especificadas en el descriptor.
	SIGNAL interficie : std_logic_vector(0 to 7);		-- interficie actual seleccionada dentro de las especificadas en el descriptor.
	SIGNAL endpoint : std_logic_vector(0 to 7);			-- endpoint actual seleccionado dentro de los especificados en el descriptor.
	SIGNAL status_dispositivo : std_logic_vector(0 to 15);	-- registro de estado del dispositivo.
	SIGNAL status_interficie : std_logic_vector(0 to 15);	-- registro de estado de la interficie.
	SIGNAL status_endpoint : std_logic_vector(0 to 15);		-- registro de estado del endpoint.
			
BEGIN
	PROCESS(clk)
		BEGIN
		IF rising_edge(clk) THEN
			CASE estat_act IS					-- El estado suspended se controla desde el modulo de capa fisica.
				WHEN attached =>
					IF vtrim = '1' THEN			-- Cuando el bus proporciona voltaje el chip activa una seal regulada.
						estat_act <= powered;			
					END IF;							
				WHEN powered =>
					led_2 <= '0';
					led_3 <= '0';
					led_4 <= '0';
					ep_type <= ep_control;		-- De momento solo se usa la transmisin de control.
					space_avail <= '1';			-- el valor 1 indica que si hay espacio disponible para tratar datos.
					timeout <= '0';				-- De momento no se controla el timeout, la transmisin de control esta muy definida.
					data_avail <= '1';
					IF reset = '1' THEN			-- seal para el reseteo del dispositivo que proviene de la capa fsica
						estat_act <= default;	-- lo traslada al estado "default"
					END IF;
				WHEN default =>
					led_2 <= '1';				-- se dejan los leds controlando el estado.
					led_3 <= '0';
					led_4 <= '0';
					IF reset = '1' THEN			
						estat_act <= default;				
					ELSIF dir_valida = '1' THEN	-- seal que indica el final de la transaccin que direcciona el dispositivo.
						estat_act <= address;
					END IF;			
				WHEN address =>
					led_3 <= '1';
					led_4 <= '0';
					IF reset = '1' THEN
						estat_act <= default;
					ELSIF dir_no_valida = '1' THEN	--seal que indica que un dispositivo ha sido "reseteado" por paquete
						estat_act <= default;
					ELSIF configurado = '1' THEN	-- seal que indica que el dispositivo ha recibido el paquete para su configuracin.
						estat_act <= configured;
					END IF;					
				WHEN configured =>
					led_4 <= '1';
					IF reset = '1' THEN
						estat_act <= default;
					ELSIF no_configurado = '1' THEN	-- seal que indica que el dispositivo ya no est configurado.
						estat_act <= address;
					END IF;					
			END CASE;
		END IF;
	END PROCESS;

	PROCESS(clk, pr)						-- si no funciona con clk quiza lo haga con la seal pr.
		BEGIN
		IF dentro = '1' THEN				-- ayuda a inicializar la seal de envio de paquete.
			dentro <= '0';
			rsac <= '0';					-- seal para el envio de paquete en su flanco de subida.
		ELSIF rising_edge(pr) THEN			-- En el flanco de subida de la seal pr se efectua el cambio de estado.
			dentro <= '1';
-----------------------------------------------------------------------------------------------------------------------------------
-- Recepcion del primer paquete de la etapa. TOKEN o SETUP
-----------------------------------------------------------------------------------------------------------------------------------			
			IF mak_est = packet_ready THEN	-- En este estado se espera un paquete que inicie una fase.
				dir_valida <= '0';			-- En cada transaccion se inicializan a 0 estas cuatro variable, 
				dir_no_valida <= '0';		-- si alguna se pone a 1 cambiar su estado la mquina general
				configurado <= '0';
				no_configurado <= '0';
				IF (estat_act = default AND wvalue(0 to 10) = DEFAULT_DIR) OR ((estat_act = address OR estat_act = configured) AND wvalue(0 to 10) = direccion_act(0 to 10)) THEN
											-- El dispositivo solo puede responder transacciones en estos estados.
											-- En el estado por defecto la direccin por defecto es "00000000000"
					IF ep_type = ep_control THEN			-- Aqui se podrian comparar los endpoints definidos segun su tipo.
						IF pide = token_out THEN			-- El pid del paquete recibido 
															-- (seal token.PID en el diagrama de estados de la especificacin).
							IF fasesc = datain_ctrl THEN	-- segun la etapa anterior de la transaccin y el tipo de paquete seguir en la misma o pasar al status.
								fasesc <= status_ctrl;
							END IF;
							mak_est <= dev_wait_Odata;		-- En un token out el siguiente paso es esperar el paquete correspondiente de datos.
															-- Eso se realiza en la etapa dev_wait_Odata.
							token_pid <= token_out;			-- Indicar el paquete anterior durante la recepcin del siguiente paquete.
						ELSIF pide = setup THEN				-- El pid del paquete recibido 
							fasesc <= setup_ctrl;			-- La etapa inicial de cualquier transaccin de control, pues el PID SETUP es el paquete inicial.
							anterior <= INT00IN;			-- Inicializa el tipo de paquete para al final saber de que transaccin se trataba, se usa en el direccionamiento.
							mak_est <= dev_wait_Odata;		-- En un setup el siguiente paso es esperar el paquete correspondiente de datos.
															-- Eso se realiza en la etapa dev_wait_Odata.
							token_pid <= setup;				-- Indicar el paquete anterior durante la recepcin del siguiente paquete.
						ELSIF pide = token_in THEN			-- El pid del paquete recibido 
							IF fasesc = dataout_ctrl THEN	-- segun la etapa anterior de la transaccin y el tipo de paquete seguir en la misma o finalizar la transaccin.
								fasesc <= setup_ctrl;	
								IF ep_trouble = '1' THEN	-- Se pone a uno cuando ha habido algn problema.
									pids <= stall;			-- Paquete de handshake. Indica problema en la comunicacin.
									ep_trouble <= '0';	
								ELSE						-- Se envia el paquete de datos de longitud zero de la fase de status.
									pids <= data1;
								END IF;
							    long_dades <= 0;									
							ELSE
															-- Se envia el paquete de handshake correspondiente.										
								IF ep_trouble = '1' THEN	-- Se pone a uno cuando ha habido algn problema.
									pids <= stall;			-- (accin Issue_packet(STALL) en el diagrama de estados de la especificacin).
									ep_trouble <= '0';
								ELSIF data_avail = '1' THEN
									IF toggle = '1' THEN	-- durante todas las etapas de la transaccin se alternan paquetes data0 y data1 el toggle lo controla.
										pids <= data1;		-- (accin Issue_packet(datax) en el diagrama de estados de la especificacin).
									ELSIF toggle = '0' THEN
										pids <= data0;
									END IF;																	
								ELSIF data_avail = '0' THEN
									pids <= nack;			-- (accin Issue_packet(NAK) en el diagrama de estados de la especificacin).
								END IF;

								toggle <= not toggle;

								IF primer = '0' THEN		-- En el primer envio el host realizar un reset tras los 8 primeros bytes.
									long_dades <= 63; 
									primer <= '1';
								ELSE
									long_dades <= (lengthw * 8) - 1;	-- longitud del paquete a enviar en bits.
								END IF;
							END IF;						
							rsac <= '1';					-- seal de enviar. Con la longitud y los datos que se cargan en el buffer.
							mak_est <= dev_resp;			-- Pasa al estado dev_resp en el que se espera un paquete de handshake.
						END IF;
---------------------------------------------------------------------------------------------------------------------------------------
--					En este lugar deberian implementarse el resto de tipos de transaccin, como los endpoints los define el dispositivo
--					es posible que solo hiciera falta en cada caso comprobar qu endpoint llega conocido ya su tipo.
--					Estos paquetes iniciales nos enviaran a otros estados diferentes a los conocidos dev_wait_Odata o dev_resp
---------------------------------------------------------------------------------------------------------------------------------------						
--					ELSIF ep_type = ep_isochronous THEN		-- no tratamos la transaccin asncrona de momento.
--						mak_est <= DEV_DO_IsochI;
--					ELSIF ep_type = ep_bulk THEN			-- no tratamos la transaccin en masa de momento.
--					ELSIF ep_type = ep_interrupt THEN		-- no tratamos la transaccin por interrupciones de momento.
					ELSE																	
						null;								-- Se ignoran los pide = sof, peticiones incorrectas o a otros dispositivos.						
					END IF;
				END IF;
-----------------------------------------------------------------------------------------------------------------------------------
-- Recepcion del segundo paquete de la etapa. DATA0 o DATA1
-----------------------------------------------------------------------------------------------------------------------------------			
			ELSIF mak_est = dev_wait_Odata THEN	
				IF estat_act = default OR estat_act = address OR estat_act = configured THEN
					IF token_pid = setup and pide = data0 and crc = '1' and space_avail = '1' THEN	-- 	pide es el tipo de paquete actual 
																									-- (seal HSD2.PID en el diagrama de estados de la especificacin).
																									-- crc es la comprobacin de errores
																									-- (seal HSD2.CRC16 en el diagrama de estados de la especificacin).
						toggle <= '1';																--  toggle ser uno en este momento, paquete datos de la fase de setup.
						long_dades <= 0;										-- En este momento responder el handshake de longitud 0
																				-- Pero ya cargar los datos en el buffer para la siguiente etapa.	
						mak_est <= packet_ready;	
						IF bmRequestType = CONV_STD_LOGIC_VECTOR(0,8) THEN	
							IF bRequest = CLEAR_FEATURE THEN	
								-- Aconsejan en el protocolo retrasar el mandar algunos errores que encontramos 
								-- en la fase de setup hasta la fase de datos, utilizare ep_trouble para ello.
								pids <= ackk;
								rsac <= '1';
								fasesc <= dataout_ctrl;							-- Estamos en un tipo de transaccin de envio de datos por parte del host
																				-- Al cambiar de sentido estaremos en la etapa de estatus.
									
								-- Las dos unicas caractersticas standard son DEVICE_REMOTE_WAKEUP para un dispositivo 
								-- recibiendo valor 1 y ENDPOINT_HALT para un punto final recibiendo valor 0, el dispositivo
								-- debe corresponderse con el de la descripcion.
								-- El valor 2 es TEST_MODE para un dispositivo pero no se le puede hacer una CLEAR_FEATURE.
								-- Tanto en el caso de TEST_MODE como en el caso de recibir una peticion a una interface 
								-- o endpoint inexistentes se devuelve un error.
								-- Falta definir el descriptor de caracteristicas.
								IF  wvalue = ZERO15 and wIndex = ZERO15 THEN 
									status_dispositivo(0 to 7) <= INT01IN;
									status_dispositivo(8 to 15) <= INT00IN;
								ELSE
									ep_trouble <= '1';
								END IF;
							ELSIF bRequest = SET_ADDRESS THEN					-- Paquete que direcciona el dispositivo				
								pids <= ackk;		
								rsac <= '1';			
								fasesc <= dataout_ctrl;		
								IF estat_act = default or estat_act = address THEN
									anterior <= SET_ADDRESS;					-- No debe hacerse efectivo aun, aqu se lo indicamos a la siguiente fase.
									direccion_act(0 to 10) <= wValue(0 to 10);	-- se recoge el valor de la direccin.
									status_dispositivo(0 to 7) <= INT01IN;		-- en algun momento hay que inicializar el bit 0 a '1', indica self-powered, al principio se pasa por aqu.	
									status_dispositivo(8 to 15) <= INT00IN;		-- en el reset del device hay que resetear el bit 1 que indica con el '0' remote wake up disabled
																				-- para direccionar el dispositivo hay que pasar por aqui siempre
									status_interficie(0 to 15) <= ZERO15;											
								END IF;
							ELSIF bRequest = SET_CONFIGURATION THEN				-- Paquete que da el ID de la configuracin a elegir
								pids <= ackk;
								rsac <= '1';	
								fasesc <= dataout_ctrl;	
								anterior <= SET_CONFIGURATION;					-- No lo hacemos efectivo aun, aqu se lo indicamos a la siguiente fase.
								IF wValue(0 to 7) = INT00IN THEN				-- Si el ID es zero se desconfigura.
									no_configurado <= '1';
									status_endpoint(0 to 15) <= ZERO15;							    	
								ELSIF wValue(0 to 7) = INT01IN THEN 
									configurado <= '1';
									status_endpoint(0 to 15) <= ZERO15;							    	
									configuracion(0 to 7) <= wValue(0 to 7);	-- no es una tonteria si hubiese varias
								ELSE
									ep_trouble <= '1';
								END IF;
							ELSIF bRequest = SET_DESCRIPTOR THEN 				-- opcional. para modificar descriptores o aadir nuevos		
								pids <= ackk;
								rsac <= '1';	
							ELSIF bRequest = SET_FEATURE THEN					-- Paquete para modificar los registros de estado segn la especificacin.
								pids <= ackk;
								rsac <= '1';	
								fasesc <= dataout_ctrl;									
								IF  wvalue = ZERO15 and wIndex = ZERO15 THEN   -- En estado address solo se aceptan los por defecto 0.
									status_dispositivo(0 to 7) <= INT03IN;
									status_dispositivo(8 to 15) <= INT00IN;
								ELSE
									ep_trouble <= '1';
								END IF;								
							ELSE
								pids <= ackk;
								rsac <= '1';	
							END IF;
						ELSIF bmRequestType = SETUNO THEN	
							IF bRequest = CLEAR_FEATURE THEN		
								pids <= ackk;
								rsac <= '1';
								fasesc <= dataout_ctrl;																	
									-- solo se utiliza una interface con el ID INT01IN, definida en el device descriptor.
								IF wvalue = ZERO15 and wIndex(0 to 7) = INT01IN and estat_act = configured THEN 	
									status_dispositivo(0) <= '1'; 
								ELSE
									ep_trouble <= '1';
								END IF;
							ELSIF bRequest = SET_FEATURE THEN	
								pids <= ackk;
								rsac <= '1';	
								fasesc <= dataout_ctrl;									
								IF  wvalue = ZERO15 and wIndex(0 to 7) = INT01IN and estat_act = configured THEN  
									-- en estado address solo se aceptan el por defecto wvalue 0.
									status_dispositivo(0) <= '1';
								ELSE
									ep_trouble <= '1';
								END IF;											
							ELSIF bRequest = SET_INTERFACE THEN	
								-- wvalue seria el alternate setting de la interface, la zero es la unica que tenemos y sin alternativas
								-- por lo que a esta peticion responderemos siempre con error, por eso la primera rama del condicional								
								-- esto forma parte intrinseca del dispositivo y sus descriptores, si se cambian tambien hay que tocar aqui.
								pids <= ackk;										
								rsac <= '1';	
								fasesc <= dataout_ctrl;				
								status_endpoint(0 to 15) <= ZERO15;		
								IF 1 = 1 THEN
									ep_trouble <= '1';			    	
								ELSIF estat_act = default THEN
									null;			    	
								ELSIF estat_act = address THEN 	
									ep_trouble <= '1';
								ELSIF wIndex = ZERO15 and estat_act = configured THEN -- wvalue tambien tendria que compararse con las posibles alternativas									
									null; -- cambiaria a la interface seleccionada
								ELSE
									ep_trouble <= '1';
								END IF;
							ELSE
								pids <= ackk;
								rsac <= '1';	
							END IF;																	
						ELSIF bmRequestType = SETDOS THEN		
							IF bRequest = CLEAR_FEATURE THEN		
								pids <= ackk;
								rsac <= '1';	
								lengthw <= 0;
								fasesc <= dataout_ctrl;										
								IF 1 = 1 THEN	-- No hay definidas caracteristicas (FEATURES) para endpoints.
									ep_trouble <= '1';
								ELSIF wvalue = ZERO15 and wIndex(0 to 7) = INT01IN and estat_act = configured THEN 
									-- el INT01IN representa la direccion en el descriptor de un hipotetico endpoint,
									status_endpoint(0 to 15) <= ZERO15;							    	
								ELSE
									ep_trouble <= '1';
								END IF;								
							ELSIF bRequest = SET_FEATURE THEN	
								pids <= ackk;
								rsac <= '1';	
								fasesc <= dataout_ctrl;									
								IF  wvalue = ZERO15 and wIndex(0 to 7) = INT01IN and estat_act = configured THEN  -- en estado address solo se aceptan los por defecto 0.
									status_dispositivo(0) <= '1';
									status_endpoint(0) <= '1';
								ELSE
									ep_trouble <= '1';
								END IF;										
							ELSE
								pids <= ackk;
								rsac <= '1';	
							END IF;																		
						ELSIF bmRequestType = GETZERO THEN		
							IF bRequest = GET_CONFIGURATION THEN	
								pids <= ackk;							
								rsac <= '1';
								lengthw <= 1;	-- es una peticin de 1 byte de datos, en caso de no serlo el comportamiento no esta definido				
								fasesc <= datain_ctrl;		
								IF estat_act = address THEN
									buffer_salida(0 to 7) <= INT00IN;
								ELSE
									buffer_salida(0 to 7) <= configuracion;									
								END IF;
							ELSIF bRequest = GET_DESCRIPTOR THEN	-- Peticin de descriptores. Aqu se definen sus valores
								pids <= ackk;
								rsac <= '1';	
								fasesc <= datain_ctrl;
								anterior <= GET_DESCRIPTOR;	
							    IF wvalue = UNO15 THEN 				-- UNO15 = DESCRIPTOR DE DISPOSITIVO
									if length >= 18 then			-- 18 bytes es su longitud total pero se puede devolver solo un parte.
										lengthw <= 18; 
									elsif length < 18 then
										lengthw <= length;																	
							    	end if;										
									buffer_salida(0 to 7) <= INT18IN;		-- Longitud del descriptor del dispositivo
									buffer_salida(8 to 15) <= INT01IN;
									buffer_salida(16 to 23) <= INT01IN; 	-- protocolo usb  01.10
									buffer_salida(24 to 31) <= INT10IN; 
									buffer_salida(32 to 39) <= INT00IN;
									buffer_salida(40 to 47) <= INT00IN;
									buffer_salida(48 to 55) <= INT00IN;
									buffer_salida(56 to 63) <= INT40IN;
									buffer_salida(64 to 71) <= INT01IN; 	-- idvendor low
									buffer_salida(72 to 79) <= INT00IN; 	-- idvendor high						
									buffer_salida(80 to 87) <= INT01IN; 	-- idproduct low
									buffer_salida(88 to 95) <= INT00IN; 	-- idproduct high
									buffer_salida(96 to 103) <= INT00IN; 	-- bdcDevice low
									buffer_salida(104 to 111) <= INT00IN;	-- bdcDevice high
									buffer_salida(112 to 119) <= INT01IN; 	-- iManufacturer
									buffer_salida(120 to 127) <= INT02IN; 	-- iProduct
									buffer_salida(128 to 135) <= INT03IN; 	-- iSerialNumber
									buffer_salida(136 to 143) <= INT01IN; 	-- bNumConfigurations
										
							    ELSIF wvalue = DOS15 THEN 	-- DOS15 = CONFIGURATION DESCRIPTOR
									if length >= 32 then	-- 32 bytes es su longitud total pero se puede devolver solo un parte.
										lengthw <= 32; 
									elsif length < 32 then
										lengthw <= length;																	
							    	end if;
--														Aqu se muestra toda la configuracion, otras demandas de
--														interface y endpoint se refieren a cual esta activa dentro de las
--														posibles.
--														Todos los campos se explican con detalle en la memoria y en la especificacin del USB.
									buffer_salida(0 to 7) <= INT09IN; 		-- longitud del descriptor de configuracion
									buffer_salida(8 to 15) <= INT02IN;
									buffer_salida(16 to 23) <= INT32IN; 	-- longitud de la suma de todos los descriptores enviados con esta peticin.
									buffer_salida(24 to 31) <= INT00IN;  
									buffer_salida(32 to 39) <= INT01IN;
									buffer_salida(40 to 47) <= INT01IN;
									buffer_salida(48 to 55) <= INT04IN;  	-- iConfiguration
									buffer_salida(56 to 63) <= INT01IN;
									buffer_salida(64 to 71) <= INT00IN;
									
									buffer_salida(72 to 79) <= INT09IN; 	-- longitud del descriptor de interficie
									buffer_salida(80 to 87) <= INT04IN;
									buffer_salida(88 to 95) <= INT00IN;
									buffer_salida(96 to 103) <= INT00IN;
									buffer_salida(104 to 111) <= INT02IN;  	-- numero de endpoints								
									buffer_salida(112 to 119) <= INT00IN;
									buffer_salida(120 to 127) <= INT00IN;
									buffer_salida(128 to 135) <= INT00IN;
									buffer_salida(136 to 143) <= INT05IN; 	-- iInterface  
									
									buffer_salida(144 to 151) <= INT07IN;
									buffer_salida(152 to 159) <= INT05IN;
									buffer_salida(160 to 167) <= EP1;		-- Direccin y sentido del primer endpoint aparte del por defecto
									buffer_salida(168 to 175) <= INT00IN;
									buffer_salida(176 to 183) <= INT40IN;
									buffer_salida(184 to 191) <= INT00IN;
									buffer_salida(192 to 199) <= INT04IN;
									
									buffer_salida(200 to 207) <= INT07IN;
									buffer_salida(208 to 215) <= INT05IN; 
									buffer_salida(216 to 223) <= EP2;		-- Direccin y sentido del segundo endpoint 
									buffer_salida(224 to 231) <= INT00IN;
									buffer_salida(232 to 239) <= INT40IN;
									buffer_salida(240 to 247) <= INT00IN;
									buffer_salida(248 to 255) <= INT04IN;
									
							    ELSIF wvalue(8 to 15) = INT03IN THEN		-- INT03IN = STRING DESCRIPTOR
							    											-- Se utilizan para enviar cadenas de caracteres para su lectura por el hombre.
							    	IF wvalue(0 to 7) = INT00IN then		-- Dirigida esta peticin a la direccin 0 se especifican los idiomas soportados.
										if length >= 4 then					-- 4 bytes es su longitud total pero se puede devolver solo un parte.
											lengthw <= 4; 
										elsif length < 4 then
											lengthw <= length;																	
								    	end if;																    
					    				buffer_salida(0 to 7) <= ('0','0','1','1','0','0','0','0');	
										buffer_salida(8 to 15) <= ('1','1','0','0','0','0','0','0');
																			-- 0x0c0a es el ID dentro de los LANGIDs para el espaol moderno. 
																			-- Spanish (Modern Sort)
										buffer_salida(16 to 23) <= ('0','1','0','1','0','0','0','0');	-- 0a  
										buffer_salida(24 to 31) <= ('0','0','1','1','0','0','0','0');	-- 0c
									ELSE									-- Se ha definido un ejemplo de descriptor de cadena, para cualquier offset que se pida.					    		
										if length >= 18 then
											lengthw <= 18; 
										elsif length < 18 then				-- 18 bytes es su longitud total pero se puede devolver solo un parte.
											lengthw <= length;																	
								    	end if;				
					    				buffer_salida(0 to 7) <= ('0','0','1','1','0','0','0','0');
										buffer_salida(8 to 15) <= ('1','1','0','0','0','0','0','0');
										buffer_salida(16 to 23) <= ('0','0','1','0','1','0','1','0');	-- 54 T	
										buffer_salida(24 to 31) <= INT00IN;								-- byte de mayor peso.
										buffer_salida(32 to 39) <= ('1','0','0','0','0','1','1','0');	-- 61 a	
										buffer_salida(40 to 47) <= INT00IN;
										buffer_salida(48 to 55) <= ('0','1','0','0','1','1','1','0');	-- 72 r
										buffer_salida(56 to 63) <= INT00IN;
										buffer_salida(64 to 71) <= ('0','1','0','1','0','1','1','0');	-- 6A j	
										buffer_salida(72 to 79) <= INT00IN;
										buffer_salida(80 to 87) <= ('1','0','0','0','0','1','1','0');	-- 61 a
										buffer_salida(88 to 95) <= INT00IN; 
										buffer_salida(96 to 103) <= ('0','0','0','0','0','1','0','0');	-- 20 (espacio)
										buffer_salida(104 to 111) <= INT00IN;								
										buffer_salida(112 to 119) <= ('1','1','1','0','0','0','1','0');	-- 47 G
										buffer_salida(120 to 127) <= INT00IN;
										buffer_salida(128 to 135) <= ('0','1','0','0','1','1','1','0');	-- 72 r	
										buffer_salida(136 to 143) <= INT00IN;   										
									END IF;
								ELSE
									led_1 <= '1'; 						-- error
							    END IF;																
							ELSIF bRequest = GET_STATUS THEN	
								led_1 <= '1';									
								pids <= ackk;
								rsac <= '1';	
								fasesc <= datain_ctrl;	
								lengthw <= 2; 	-- es una peticin siempre de 2 bytes y en caso de no serlo el comportamiento no esta definido												
								IF 1 = 1 THEN
									-- para el caso de un dispositivo el comportamiento del GETZERO no esta definido 
									-- en los casos que la informacion no concuerde, se prosigo con normalidad siempre.
									buffer_salida(0 to 15)  <= status_dispositivo(0 to 15);
								ELSE
									ep_trouble <= '1';
								END IF;
							ELSE
								led_1 <= '1';									
								pids <= ackk;
								rsac <= '1';	
							END IF;			
						ELSIF bmRequestType = GETUNO THEN		
							led_1 <= '1';	
							IF bRequest = GET_INTERFACE THEN		-- solo pide la interface activa
								pids <= ackk;
								rsac <= '1';	
								fasesc <= datain_ctrl;	
								lengthw <= 1; 	-- peticin de 1 byte, en caso de no serlo el comportamiento no esta definido	
								IF estat_act = default THEN
									buffer_salida(0 to 7) <= ('0','0','0','0','0','0','0','0');
								ELSIF wIndex = ZERO15 and estat_act = configured THEN 			-- unica interface que se usa
				    				buffer_salida(0 to 7) <= ('0','0','0','0','0','0','0','0'); -- no tiene alternate settings
								ELSE
									ep_trouble <= '1';
								END IF;
							ELSIF bRequest = GET_STATUS THEN	
								pids <= ackk;
								rsac <= '1';	
								fasesc <= datain_ctrl;	
								lengthw <= 2; 	-- length deberia serlo y en caso de no serlo el comportamiento no esta definido												
								IF estat_act = default THEN
									buffer_salida(0 to 15)  <= status_interficie(0 to 15);
								ELSIF wIndex = ZERO15 and estat_act = configured THEN
									buffer_salida(0 to 15)  <= status_interficie(0 to 15);
								ELSE
									ep_trouble <= '1';
								END IF;									
							ELSE
								pids <= ackk;
								rsac <= '1';	
--								error									
							END IF;																										
						ELSIF bmRequestType = GETDOS THEN			
							led_1 <= '1';														
							IF bRequest = GET_STATUS THEN		
								pids <= ackk;							
								rsac <= '1';	
								fasesc <= datain_ctrl;	
								lengthw <= 2; 	-- length deberia serlo y en caso de no serlo el comportamiento no esta definido												
								IF estat_act = default THEN								
									buffer_salida(0 to 15)  <= status_endpoint(0 to 15);								
								ELSIF wIndex(0 to 7) = INT01IN and wIndex(8 to 15) = INT00IN  and estat_act = configured THEN	--INT01IN representa el enpoint que haya puesto en el descriptor
									buffer_salida(0 to 15)  <= status_endpoint(0 to 15);
								ELSE
									ep_trouble <= '1';
								END IF;										
							ELSIF bRequest = SYNCH_FRAME THEN	
								-- Cuando un endpoint soporta transferencias sincronas puede pedir transferencias por-frame y debe ponerse de acuerdo con
								-- el host en que frame empezar, este numero de frame es el que se devuelve al host.
								-- Como no tenemos endpoints con comportamiento sincrono, defino el condicional respondiendo error.
								pids <= ackk;							
								rsac <= '1';	
								fasesc <= datain_ctrl;		
								lengthw <= 2;			
								IF 1 = 1 THEN
									ep_trouble <= '1';
								ELSIF estat_act = address THEN
									ep_trouble <= '1';
								ELSIF wIndex(0 to 7) = INT01IN and estat_act = configured THEN
									null;	-- devolveriamos el frame number, poniendolo en el buffer
								ELSE
									ep_trouble <= '1';
								END IF;
							END IF;		
						ELSE
							led_1 <= '1';	--	error									
						END IF;
											-- A parte de las peticiones hay otra posibilidades en este estado
											-- Se pueden recibir paquetes tras un token out.
					ELSIF token_pid = token_out and ep_trouble = '1' THEN		-- en caso de haber un error en la transmisin se manda un stall.
						toggle <= '0';											-- se resetea el toggle.
						pids <= stall;											-- pid del paquete de handshake.
						rsac <= '1';											-- seal de envio.
						long_dades <= 0;										-- longitud de los datos del paquete.
						mak_est <= packet_ready;
						fasesc <= setup_ctrl;									-- se resetea la fase.
						ep_trouble <= '0';										-- se resetea el estado del canal.
					ELSIF token_pid = token_out and pide = data1 and fasesc = status_ctrl and crc = '1' and space_avail = '1' THEN  						
						toggle <= '0';											-- En la fase de status el paquete de datos siempre es un data1
						pids <= ackk;											-- Si todo concuerda se responde un ACK. Correcto.
						rsac <= '1';		
						long_dades <= 0;								
						mak_est <= packet_ready;		
						fasesc <= setup_ctrl;									-- se resetea la etapa
						anterior <= INT00IN;									-- se resetea el tipo de peticin
					ELSIF token_pid = token_out and ((pide = data0 and toggle = '0') or (pide = data1 and toggle = '1')) and crc = '1' and space_avail = '1' THEN  
						toggle <= not toggle;									-- durante una transmisin deben coincidir el toggle con el paquete de datos, pues
																				-- representa el que corresponde recibir
																				-- (dataX es la seal HSD2.x en los diagramas de estados de la especificacin)
																				-- Dev_accept_data es como se le llama a este paso en los diagramas de la especificacin
						pids <= ackk;											
						rsac <= '1';											
						long_dades <= 0;								
						mak_est <= packet_ready;			
						fasesc <= dataout_ctrl;			
					ELSIF token_pid = token_out and ((pide = data0 and toggle = '0') or (pide = data1 and toggle = '1')) and crc = '1' and space_avail = '0' THEN  
						pids <= nack;											-- El NACK se responde cuando no se desean recibir aun los datos, 
																				-- por ejemplo porque no hay espacio.
						rsac <= '1';											
						long_dades <= 0;								
						mak_est <= packet_ready;
						fasesc <= dataout_ctrl;								
					ELSIF pide = sof THEN						
						null;
					ELSE
						mak_est <= packet_ready;		-- desde dev_wait_Odata siempre deberia volver excepto con un sof.
						fasesc <= setup_ctrl;		
					END IF;
				ELSE
					null;								--	error					
				END IF;
-----------------------------------------------------------------------------------------------------------------------------------
-- Espera del Handshake. Cuando la transaccin a sido TOKEN IN.
-----------------------------------------------------------------------------------------------------------------------------------			
			ELSIF mak_est = dev_resp THEN	
				IF estat_act = default OR estat_act = address OR estat_act = configured THEN
					toggle <= not toggle;			-- Aqui invertimos el toggle en el caso de un token in. 
					IF pide = ackk THEN				-- El host nos indica que le hemos mandado el pakete correctamente.
						if anterior = SET_ADDRESS and direccion_act(0 to 10) = DEFAULT_DIR and estat_act = address then
							dir_no_valida <= '1';	
						elsif anterior = SET_ADDRESS then
							dir_valida <= '1';		-- Tambin es aqu donde se hace efectivo el direccionamiento.
						end if;
						mak_est <= packet_ready;
					ELSIF pide = nack OR pide = stall OR pide = nyet OR timeout = '1' THEN
								-- El host nos indica que ha habido un inconveniente NACK o error STALL en el envio.
								-- El nyet no se usa en esta velocidad de transmisin.
						mak_est <= packet_ready;
					ELSIF pide = sof THEN		
						null;
					ELSE
						mak_est <= packet_ready;						
					END IF;
				END IF;
			ELSE			
				mak_est <= packet_ready;										
			END IF;
		END IF;		
	END PROCESS;
END arq_CtrlUnit45;
